﻿// The MIT License (MIT)
// 
// COPYRIGHT 2013. BLOOMBERG FINANCE L.P.
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System.IO;
using System.Reflection;
using System.Text;
using System.Xml;

namespace PluginHost.Hosting
{
    internal class RuntimeConfig
    {
        private const string BindingsNamespace = "urn:schemas-microsoft-com:asm.v1";
        private static readonly Encoding _encoding = Encoding.UTF8;

        private readonly XmlDocument _document;
        private readonly XmlElement _binding;

        public RuntimeConfig()
        {
            _document = new XmlDocument();
            _document.AppendChild(
                _document.CreateXmlDeclaration("1.0", _encoding.WebName, null)
                );

            _binding = _document.CreateElement("assemblyBinding", BindingsNamespace);

            _document.AppendChild(_document.CreateElement("configuration"))
                .AppendChild(_document.CreateElement("runtime"))
                .AppendChild(_binding);
        }

        public void AddCodeBaseFor(AssemblyName name)
        {
            var assemblyElement = _document
                .CreateElement("dependentAssembly", BindingsNamespace);

            var identityElement = _document
                .CreateElement("assemblyIdentity", BindingsNamespace);

            identityElement.SetAttribute("name", name.Name);
            identityElement.SetAttribute(
                "publicKeyToken", FormatPublicKeyToken(name));
            identityElement.SetAttribute(
                "culture", FormatCulture(name));

            var codeBaseElement = _document
                .CreateElement("codeBase", BindingsNamespace);

            codeBaseElement.SetAttribute(
                "version", name.Version.ToString());
            codeBaseElement.SetAttribute("href", name.CodeBase);

            assemblyElement.AppendChild(identityElement);
            assemblyElement.AppendChild(codeBaseElement);

            _binding.AppendChild(assemblyElement);
        }

        public byte[] GetBytes()
        {
            using (var stream = new MemoryStream())
            using (var writer = new StreamWriter(stream, _encoding))
            {
                _document.Save(writer);
                return stream.ToArray();
            }
        }

        private static string FormatCulture(AssemblyName assemblyName)
        {
            var culture = assemblyName.CultureInfo.Name;
            return string.IsNullOrEmpty(culture) ? "neutral" : culture;
        }

        private static string FormatPublicKeyToken(AssemblyName assemblyName)
        {
            var bytes = assemblyName.GetPublicKeyToken();

            if (bytes == null || bytes.Length == 0)
            {
                return "null";
            }
            const string hexDigits = "0123456789abcdef";
            var text = new StringBuilder(bytes.Length*2);

            foreach (var value in bytes)
            {
                text.Append(hexDigits[value >> 4]);
                text.Append(hexDigits[value & 15]);
            }
            return text.ToString();
        }
    }
}