﻿// The MIT License (MIT)
// 
// COPYRIGHT 2013. BLOOMBERG FINANCE L.P.
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using Bloomberglp.Blpapi;
using System.Diagnostics;

namespace DapiSample
{
    public class MarketDataProvider : IDisposable
    {
        private readonly Session _session;
        private readonly Service _refDataService;
       
        public event Action<List<DataPoint>> MarketDataEventHandler;
        public event Action<string> MarketDataErrorHandler;

        public MarketDataProvider()
        {
            var sessionOptions = new SessionOptions
            {
                ServerHost = "localhost",
                ServerPort = 8194,
                ClientMode = SessionOptions.ClientModeType.DAPI,
                AutoRestartOnDisconnection = true,
                ConnectTimeout = 60000
            };

            _session = new Session(sessionOptions, ProcessEventCallBack);

            if (_session.Start())
            {
                if (_session.OpenService("//blp/refdata"))
                {
                    _refDataService = _session.GetService("//blp/refdata");
                }
                else
                    Trace.WriteLine("c Reference Data Service");
            }
            else
                Trace.WriteLine("Failed to start Bloomberg Api session");
        }

        private void ProcessEventCallBack(Event eventObject, Session session)
        {
            try
            {
                if (eventObject.Type == Event.EventType.RESPONSE)
                {
                    OnProcessReferenceDataResponse(eventObject);
                    return;
                }

                foreach (var msg in eventObject)
                {
                    Trace.WriteLine(msg.MessageType.ToString());
                }
            }
            catch (Exception ex)
            {
               Trace.WriteLine(string.Format("Exception during processing Bloomberg Data API Services Event: {0}", ex), "ERROR");
            }
        }

        public void RequestReferenceData(string security, DateTime start, DateTime end, string periodicity)
        {
            Request request = _refDataService.CreateRequest("HistoricalDataRequest");
            Element securities = request.GetElement("securities");

            securities.AppendValue(security);
            Element fields = request.GetElement("fields");
            fields.AppendValue("PX_LAST");

            request.Set("periodicityAdjustment", "ACTUAL");
            request.Set("periodicitySelection", periodicity);
            
            request.Set("startDate", string.Format("{0}{1:D2}{2:D2}", start.Year, start.Month, start.Day));
            request.Set("endDate", string.Format("{0}{1:D2}{2:D2}", end.Year, end.Month, end.Day));

            Trace.WriteLine("Sending Request: " + request);
            _session.SendRequest(request, null);
        }


        private void OnProcessReferenceDataResponse(Event eventObject)
        {
            var series = new List<DataPoint>();
            foreach (Message msg in eventObject)
            {
                Trace.WriteLine(msg.AsElement);

                var element = msg.AsElement;

                var sd = element.GetElement("securityData");
                var fd = sd.GetElement("fieldData");

                if (fd.NumValues > 0)
                {
                    for (int i = 0; i < fd.NumValues; i++)
                    {
                        var val = fd.GetValue(i);
                        var price = (double)((Element)val).GetElement("PX_LAST").GetValue();
                        var dt = (Datetime)((Element)val).GetElement("date").GetValue();

                        series.Add(new DataPoint(new DateTime(dt.Year, dt.Month, dt.DayOfMonth), price));

                    }
                    if (MarketDataEventHandler != null)
                        MarketDataEventHandler(series);
                }
                else //error
                {
                    if (MarketDataErrorHandler != null)
                        MarketDataErrorHandler(sd.GetElement("securityError").GetElement("message").ToString() + sd.GetElement("security"));
                    Trace.WriteLine(sd.GetElement("securityError").GetElement("message").ToString() + sd.GetElement("security"));
                }
            }
        }

        public void Dispose()
        {
            _session.Stop();
            ((IDisposable)_session).Dispose();
        }
    }
}
